;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Package: (C-INTERFACE); -*-

;;; C-Interface
;;; Copyright (C) 2002 Matthias Hlzl

;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 2 of the
;;; License, or (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;;; 02111-1307, USA.

(in-package "C-INTERFACE")

(define-fixture c-type-fixture ()
  ())

(defmethod set-up ((fixture c-type-fixture))
  (declare (ignorable fixture))
  (clear-c-name-translations))

(define-test-case c-type-test-case
                  :parent 'c-interface
                  :fixture-class 'c-type-fixture)

(set-current-test-case c-type-test-case)

(deftest (c-type-for/1)
  (expect (c-type-for 'char) 'c-char)
  (expect (c-type-for 'signed-char) 'signed-c-char)
  (expect (c-type-for 'unsigned-char) 'unsigned-c-char))

(deftest (c-type-for/2)
  (expect (c-type-for 'int ) 'c-int)
  (expect (c-type-for 'signed-int ) 'signed-c-int)
  (expect (c-type-for 'unsigned-int ) 'unsigned-c-int))

(deftest (c-type-for/3)
  (expect (c-type-for 'long ) 'c-long)
  (expect (c-type-for 'signed-long ) 'signed-c-long)
  (expect (c-type-for 'unsigned-long ) 'unsigned-c-long))

(deftest (c-type-for/4)
  (expect (c-type-for 'float ) 'c-float)
  (expect (c-type-for 'double ) 'c-double)
  (expect (c-type-for 'void ) 'c-void))

(deftest (c-type-for/5)
  (expect (c-type-for 'typedef) 'c-typedef-reference))

(deftest (c-type-for/6)
  (expect (c-type-for 'struct-ref) 'c-struct-type-reference))

(deftest (c-type-for/7)
  (expect (c-type-for 'union-ref) 'c-union-type-reference))

(deftest (c-type-for/8)
  (expect (c-type-for 'enum-ref) 'c-enum-type-reference))

(deftest (c-type-for/9)
  (expect (c-type-for 'pointer) 'c-pointer-type-reference))

(deftest (c-type-for/10)
  (expect (c-type-for 'function) 'c-function-type-reference))

(deftest (c-type-for/11)
  (expect (c-type-for 'array) 'c-array-type-reference))

(deftest (convert-ffi-type/1)
  (expect-type (convert-ffi-type '(char ())) 'c-char)
  (expect-type (convert-ffi-type '(signed-char ())) 'signed-c-char)
  (expect-type (convert-ffi-type '(unsigned-char ())) 'unsigned-c-char))

(deftest (convert-ffi-type/2)
  (expect-type (convert-ffi-type '(int ())) 'c-int)
  (expect-type (convert-ffi-type '(signed-int ())) 'signed-c-int)
  (expect-type (convert-ffi-type '(unsigned-int ())) 'unsigned-c-int))

(deftest (convert-ffi-type/3)
  (expect-type (convert-ffi-type '(long ())) 'c-long)
  (expect-type (convert-ffi-type '(signed-long ())) 'signed-c-long)
  (expect-type (convert-ffi-type '(unsigned-long ())) 'unsigned-c-long))

(deftest (convert-ffi-type/3a)
  (expect-type (convert-ffi-type '(long-long ())) 'c-long-long)
  (expect-type (convert-ffi-type '(signed-long-long ())) 'signed-c-long-long)
  (expect-type (convert-ffi-type '(unsigned-long-long ())) 'unsigned-c-long-long))

(deftest (convert-ffi-type/4)
  (expect-type (convert-ffi-type '(float ())) 'c-float)
  (expect-type (convert-ffi-type '(double ())) 'c-double)
  (expect-type (convert-ffi-type '(void ())) 'c-void))

(deftest (convert-ffi-type/5)
  (let ((typedef (convert-ffi-type '(typedef "GLXPbufferClobberEvent"))))
    (expect-type typedef 'c-typedef-reference)
    (expect (lisp-name typedef) 'glx-pbuffer-clobber-event)))

(deftest (convert-ffi-type/6)
  (let ((struct (convert-ffi-type '(struct-ref "_XDisplay"))))
    (expect-type struct 'c-struct-type-reference)
    (expect (lisp-name struct) '%x-display)))

(deftest (convert-ffi-type/7)
  (let ((union (convert-ffi-type '(union-ref "__GLXEvent"))))
    (expect-type union 'c-union-type-reference)
    (expect (lisp-name union) '%%glx-event)))

(deftest (convert-ffi-type/8)
  (let ((enum (convert-ffi-type '(enum-ref "myEnum"))))
    (expect-type enum 'c-enum-type-reference)
    (expect (lisp-name enum) 'my-enum)))

(deftest (convert-ffi-type/9)
  (let ((pointer (convert-ffi-type '(pointer (char ())))))
    (expect-type pointer 'c-pointer-type-reference)
    (expect-type (base-type-reference pointer) 'c-char)))

(deftest (convert-ffi-type/10)
  (let* ((pointer (convert-ffi-type '(pointer (typedef "Display"))))
	 (base-type (base-type-reference pointer)))
    (expect-type pointer 'c-pointer-type-reference)
    (expect-type base-type 'c-typedef-reference)
    (expect (lisp-name base-type) 'display)))

(deftest (convert-ffi-type/11)
  (let ((function (convert-ffi-type '(function 
				     ((pointer (struct-ref "_XDisplay")))
				     (typedef "XID")))))
    (expect-type function 'c-function-type-reference)
    (let ((arg (first (arglist function))))
      (expect-type arg 'c-pointer-type-reference)
      (let ((base-type (base-type-reference arg)))
	(expect-type base-type 'c-struct-type-reference)
	(expect (lisp-name base-type) '%x-display)))
    (let ((res (result-type function)))
      (expect-type res 'c-typedef-reference)
      (expect (lisp-name res) 'xid))))
 
(deftest (convert-ffi-type/12)
  (let ((array (convert-ffi-type '(array 24 (long ())))))
    (expect-type array 'c-array-type-reference)
    (expect-type (base-type-reference array) 'c-long)
    (expect (array-size array) 24)))

(deftest (convert-ffi-type/13)
  (let ((array (convert-ffi-type '(array 0 (long ())))))
    (expect-type array 'c-array-type-reference)
    (expect-type (base-type-reference array) 'c-long)
    (expect (array-size array) 0)))



#||
(show-results (run-tests 'c-interface))
||#
