;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Package: (C-INTERFACE); -*-

;;; C-Interface
;;; Copyright (C) 2002 Matthias Hlzl

;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 2 of the
;;; License, or (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;;; 02111-1307, USA.

(in-package "C-INTERFACE")

(define-fixture cmucl-fixture ()
  ())

(defmethod set-up ((fixture cmucl-fixture))
  (declare (ignorable fixture))
  (clear-all-ffi-tables))

(define-test-case cmucl-test-case
                  :parent 'c-interface
                  :fixture-class 'cmucl-fixture)

(set-current-test-case cmucl-test-case)


(deftest (convert/function/1)
  (expect (handler-bind ((warning #'muffle-warning))
	    (convert-form '(function ("/usr/share/doc/NVIDIA_GLX-1.0/include/GL/glx.h" 132) 
			    "glXQueryMaxSwapBarriersSGIX" 
			    (function 
			     ((pointer (typedef "Display")) (int ()) (pointer (int ())))
			     (int ())) (extern))))
	  '(defun glx-query-max-swap-barriers-sgix (arg-0 arg-1 arg-2)
	    (alien:alien-funcall
	     (alien:extern-alien "glXQueryMaxSwapBarriersSGIX"
	      (function c-call:int (* display) c-call:int
			(* c-call:int)))
	     arg-0
	     arg-1
	     arg-2))
	  :test #'equal))

(deftest (convert/variable/1)
  (expect (convert-form '(var ("test.c" 2) "j" (int ())))
	  '(alien:def-alien-variable (j "j") c-call:int)
	  :test #'equal))

(deftest (convert/typedef/1)
  (expect (handler-bind ((warning #'muffle-warning))
	    (convert-form '(type ("/usr/share/doc/NVIDIA_GLX-1.0/include/GL/glx.h" 156) 
			    "GLXEvent" 
			    (union-ref "__GLXEvent"))))
	  '(alien:def-alien-type glx-event (union %%glx-event))
	  :test #'equal))

(deftest (convert/struct/1)
  (expect (handler-bind ((warning #'muffle-warning))
	    (convert-form
	     '(struct ("/usr/share/doc/NVIDIA_GLX-1.0/include/GL/glx.h" 139) 
	       "91" 
	       (("event_type" (field (int ()) 0 4))
		("draw_type" (field (int ()) 4 4))
		("serial" (field (unsigned-long ()) 8 4))
		("send_event" (field (int ()) 12 4))
		("display" (field (pointer (typedef "Display")) 16 4))
		("drawable" (field (typedef "GLXDrawable") 20 4))
		("buffer_mask" (field (unsigned ()) 24 4))
		("aux_buffer" (field (unsigned ()) 28 4))
		("x" (field (int ()) 32 4))
		("y" (field (int ()) 36 4))
		("width" (field (int ()) 40 4))
		("height" (field (int ()) 44 4))
		("count" (field (int ()) 48 4))))))
	  '(alien:def-alien-type nil
	      (alien:struct anonymous-91
		      (event-type c-call:int)
		      (draw-type c-call:int)
		      (serial c-call:unsigned-long)
	              (send-event c-call:int)
		      (display (alien:* display))
		      (drawable glx-drawable)
		      (buffer-mask c-call:unsigned-int)
		      (aux-buffer c-call:unsigned-int)
		      (x c-call:int)
		      (y c-call:int)
		      (width c-call:int)
		      (height c-call:int)
		      (count c-call:int)))
	  :test #'equal))

(deftest (convert/union/1)
  (expect (handler-bind ((warning #'muffle-warning))
	    (convert-form
	     '(union ("/usr/include/X11/Xlib.h" 966) 
	       "_XEvent" 
	       (("type" (field (int ()) 0 4))
		("xany" (field (typedef "XAnyEvent") 0 20))
		("xkey" (field (typedef "XKeyEvent") 0 60))
		("xbutton" (field (typedef "XButtonEvent") 0 60))
		("xmotion" (field (typedef "XMotionEvent") 0 60))
		("xcrossing" (field (typedef "XCrossingEvent") 0 68))
		("xfocus" (field (typedef "XFocusChangeEvent") 0 28))
		("xexpose" (field (typedef "XExposeEvent") 0 40))
		("xgraphicsexpose" (field (typedef "XGraphicsExposeEvent") 0 48))
		("xnoexpose" (field (typedef "XNoExposeEvent") 0 28))
		("xvisibility" (field (typedef "XVisibilityEvent") 0 24))
		("xcreatewindow" (field (typedef "XCreateWindowEvent") 0 48))
		("xdestroywindow" (field (typedef "XDestroyWindowEvent") 0 24))
		("xunmap" (field (typedef "XUnmapEvent") 0 28))
		("xmap" (field (typedef "XMapEvent") 0 28))
		("xmaprequest" (field (typedef "XMapRequestEvent") 0 24))
		("xreparent" (field (typedef "XReparentEvent") 0 40))
		("xconfigure" (field (typedef "XConfigureEvent") 0 52))
		("xgravity" (field (typedef "XGravityEvent") 0 32))
		("xresizerequest" (field (typedef "XResizeRequestEvent") 0 28))
		("xconfigurerequest" (field (typedef "XConfigureRequestEvent") 0 56))
		("xcirculate" (field (typedef "XCirculateEvent") 0 28))
		("xcirculaterequest" (field (typedef "XCirculateRequestEvent") 0 28))
		("xproperty" (field (typedef "XPropertyEvent") 0 32))
		("xselectionclear" (field (typedef "XSelectionClearEvent") 0 28))
		("xselectionrequest" (field (typedef "XSelectionRequestEvent") 0 40))
		("xselection" (field (typedef "XSelectionEvent") 0 36))
		("xcolormap" (field (typedef "XColormapEvent") 0 32))
		("xclient" (field (typedef "XClientMessageEvent") 0 48))
		("xmapping" (field (typedef "XMappingEvent") 0 32))
		("xerror" (field (typedef "XErrorEvent") 0 20))
		("xkeymap" (field (typedef "XKeymapEvent") 0 52))
		("pad" (field (array 24 (long ())) 0 96))))))
	  '(alien:def-alien-type nil ; %x-event
	      (alien:union %x-event
		     (type c-call:int)
		     (xany x-any-event)
		     (xkey x-key-event)
		     (xbutton x-button-event)
		     (xmotion x-motion-event)
		     (xcrossing x-crossing-event)
		     (xfocus x-focus-change-event)
		     (xexpose x-expose-event)
		     (xgraphicsexpose x-graphics-expose-event)
		     (xnoexpose x-no-expose-event)
		     (xvisibility x-visibility-event)
		     (xcreatewindow x-create-window-event)
		     (xdestroywindow x-destroy-window-event)
		     (xunmap x-unmap-event)
		     (xmap x-map-event)
		     (xmaprequest x-map-request-event)
		     (xreparent x-reparent-event)
		     (xconfigure x-configure-event)
		     (xgravity x-gravity-event)
		     (xresizerequest x-resize-request-event)
		     (xconfigurerequest x-configure-request-event)
		     (xcirculate x-circulate-event)
		     (xcirculaterequest x-circulate-request-event)
		     (xproperty x-property-event)
		     (xselectionclear x-selection-clear-event)
		     (xselectionrequest x-selection-request-event)
		     (xselection x-selection-event)
		     (xcolormap x-colormap-event)
		     (xclient x-client-message-event)
		     (xmapping x-mapping-event)
		     (xerror x-error-event)
		     (xkeymap x-keymap-event)
		     (pad (array c-call:long 24))))
	  :test #'equal))

(deftest (convert/macro/1)
  (expect (convert-form
	   '(macro
	     ("/usr/share/doc/nvidia_glx-1.0/include/gl/gl.h" 1784)
	     "gl_offset_texture_matrix_nv" "0x86e1"))
	  '(defconstant +gl-offset-texture-matrix-nv+ 34529)
	  :test #'equal))

(deftest (convert/enum/1)
  (expect (convert-form
	   '(enum ("/usr/include/X11/Xutil.h" 163) 
	     "85" 
	     (("XStringStyle" 0)
	      ("XCompoundTextStyle" 1)
	      ("XTextStyle" 2)
	      ("XStdICCTextStyle" 3)
	      ("XUTF8StringStyle" 4))))
	  '(alien:def-alien-type nil ; anonymous-85
	      (alien:enum  anonymous-85
		    (:x-string-style 0)
		    (:x-compound-text-style 1)
		    (:x-text-style 2)
		    (:x-std-icc-text-style 3)
		    (:xutf8-string-style 4)))
	  :test #'equal))

#||
(show-results (run-tests 'c-interface))
||#
