;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Package: (C-INTERFACE); -*-

;;; C-Interface
;;; Copyright (C) 2002 Matthias Hlzl

;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 2 of the
;;; License, or (at your option) any later version.

;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.

;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, write to the Free Software
;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;;; 02111-1307, USA.

(in-package "C-INTERFACE")

(define-fixture utilities-fixture ()
  ())

(define-fixture split-string-fixture (utilities-fixture)
  ((char-bag-1 :initform '(#\space))
   (char-bag-2 :initform '(#\a #\b))
   (char-bag-3 :initform '((#\space . :space)))
   (char-bag-4 :initform '((#\space . :space)
			   (#\newline . :newline)))
   (ab-string :initform "abba")
   (abc-string :initform "abccba")
   (cba-string :initform "cbaabc")
   (two-word-string :initform "Two words")))

(define-test-case utilities-test-case
                  :parent 'c-interface
                  :fixture-class 'utilities-fixture)

(set-current-test-case utilities-test-case)

(deftest (listify)
  (expect (listify nil) nil)
  (expect (listify '(1 2 3)) '(1 2 3) :test #'equal)
  (expect (listify "String") '("String") :test #'equal))

(deftest (list-of)
  (expect (list-of 0 :foo) '())
  (expect-not (list-of 1 :foo) '(:foo))
  (expect (list-of 1 :foo) '(:foo) :test #'equal)
  (expect (list-of 3 :foo) '(:foo :foo :foo) :test #'equal))

(deftest (singleton-list-p)
  (expect-false (singleton-list-p 123))
  (expect-false (singleton-list-p ()))
  (expect-false (singleton-list-p '(1 . 2)))
  (expect-true (singleton-list-p '(1)))
  (expect-true (singleton-list-p '((1 2))))
  (expect-false (singleton-list-p '(1 2))))

(defparameter *string-1*
"This is a
String    with
embedded newlines
and such")

(defparameter *string-1-result*
  '("This" :SPACE "is" :SPACE "a" :NEWLINE "String" :SPACE :SPACE
           :SPACE :SPACE "with" :NEWLINE "embedded" :SPACE "newlines"
           :NEWLINE "and" :SPACE "such"))

(defparameter *string-2*
"This is a
  String    with
embedded newlines     
and such")

(defparameter *string-2-result*
  '("This" :SPACE "is" :SPACE "a" :NEWLINE :SPACE :SPACE "String"
           :SPACE :SPACE :SPACE :SPACE "with" :NEWLINE "embedded"
           :SPACE "newlines" :SPACE :SPACE :SPACE :SPACE :SPACE 
           :NEWLINE "and" :SPACE "such"))

(deftest (split-string-1 :fixture split-string-fixture)
  (expect (split-string char-bag-1 "") '("") :test #'equal)
  (expect (split-string char-bag-2 "") '("") :test #'equal)
  (expect (split-string char-bag-3 "") '("") :test #'equal)
  (expect (split-string char-bag-4 "") '("") :test #'equal))

(deftest (split-string-2 :fixture split-string-fixture)
  (expect (split-string char-bag-1 ab-string) '("abba") :test #'equal)
  (expect (split-string char-bag-2 ab-string) '() :test #'equal)
  (expect (split-string char-bag-3 ab-string) '("abba") :test #'equal))

(deftest (split-string-3 :fixture split-string-fixture)
  (expect (split-string char-bag-1 abc-string) '("abccba") :test #'equal)
  (expect (split-string char-bag-2 abc-string) '("cc") :test #'equal)
  (expect (split-string char-bag-3 abc-string) '("abccba") :test #'equal)
  (expect (split-string char-bag-4 abc-string) '("abccba") :test #'equal))

(deftest (split-string-4 :fixture split-string-fixture)
  (expect (split-string char-bag-1 cba-string) '("cbaabc") :test #'equal)
  (expect (split-string char-bag-2 cba-string) '("c" "c") :test #'equal)
  (expect (split-string char-bag-3 cba-string) '("cbaabc") :test #'equal)
  (expect (split-string char-bag-4 cba-string) '("cbaabc") :test #'equal))

(deftest (split-string-5 :fixture split-string-fixture)
  (expect (split-string char-bag-1 two-word-string) '("Two" "words") :test #'equal)
  (expect (split-string char-bag-2 two-word-string) '("Two words") :test #'equal)
  (expect (split-string char-bag-3 two-word-string) '("Two" :space "words") :test #'equal)
  (expect (split-string char-bag-4 two-word-string) '("Two" :space "words") :test #'equal))

(deftest (split-string-6 :fixture split-string-fixture)
  (expect (split-string char-bag-4 *string-1*) *string-1-result* :test #'equal)
  (expect (split-string char-bag-4 *string-2*) *string-2-result* :test #'equal))

(deftest (try-to-parse-as-number)
  (expect (try-to-parse-as-number "0") 0)
  (expect (try-to-parse-as-number "1") 1)
  (expect (try-to-parse-as-number "1L") 1)
  (expect (try-to-parse-as-number "0x1") 1)
  (expect (try-to-parse-as-number "007") 7)
  (expect (try-to-parse-as-number "0xFF") 255)
  (expect (try-to-parse-as-number "12345678901234567890") 12345678901234567890)
  (expect-false (try-to-parse-as-number ""))
  (expect-false (try-to-parse-as-number "0x"))
  (expect-false (try-to-parse-as-number "#x123"))
  (expect-false (try-to-parse-as-number "ox123"))
  (expect-false (try-to-parse-as-number "f4")))

(deftest (known-prefix/1)
  (multiple-value-bind (prefix index)
      (known-prefix "foobar")
    (expect-false prefix)
    (expect-false index)))

(deftest (known-prefix/2)
  (multiple-value-bind (prefix index)
      (known-prefix "_foobar")
    (expect prefix "%" :test #'string=)
    (expect index 1)))

(deftest (known-prefix/3)
  (multiple-value-bind (prefix index)
      (known-prefix "__")
    (expect prefix "%%" :test #'string=)
    (expect index 2)))


(deftest (unstudlify/1)
  (expect (unstudlify "") "" :test #'equal))
(deftest (unstudlify/2)
  (expect (unstudlify "A") "A" :test #'equal))
(deftest (unstudlify/3)
  (expect (unstudlify "a") "A" :test #'equal))
(deftest (unstudlify/4)
  (expect (unstudlify "ab") "AB" :test #'equal))
(deftest (unstudlify/5)
  (expect (unstudlify "Ab") "AB" :test #'equal))
(deftest (unstudlify/6)
  (expect (unstudlify "aB") "A-B" :test #'equal))
(deftest (unstudlify/7)
  (expect (unstudlify "AB") "AB" :test #'equal))
(deftest (unstudlify/8)
  (expect (unstudlify "FooBar") "FOO-BAR" :test #'equal))
(deftest (unstudlify/9)
  (expect (unstudlify "fooBar") "FOO-BAR" :test #'equal))
(deftest (unstudlify/10)
  (expect (unstudlify "fooBarBAZMac") "FOO-BAR-BAZ-MAC" :test #'equal))
(deftest (unstudlify/11)
  (expect (unstudlify "ThisIsAVeryLongNameETC")
	  "THIS-IS-A-VERY-LONG-NAME-ETC" :test #'equal))
(deftest (unstudlify/12)
  (expect (unstudlify "this_is_a_sane_name") "THIS-IS-A-SANE-NAME" :test #'equal))
(deftest (unstudlify/13)
  (expect (unstudlify "XxX" :prefix "foo") "fooXX-X" :test #'equal)
  (expect (unstudlify "YxX" :prefix "foo") "fooYX-X" :test #'equal))
(deftest (unstudlify/14)
  (expect (unstudlify "ABc" :postfix "def") "A-BCdef" :test #'equal))
(deftest (unstudlify/15)
  (expect (unstudlify "xX" :prefix "+" :postfix "+") "+X-X+" :test #'equal))

(deftest (c-translations)
  (clear-c-name-translations)
  (expect-false (find-c-translation "foo"))
  (expect-false (find-c-translation "bar"))
  (add-c-translation "foo" 'my-foo)
  (expect (find-c-translation "foo") 'my-foo)
  (expect-false (find-c-translation "bar"))
  (define-c-translation "bar" 'your-bar)
  (expect (find-c-translation "foo") 'my-foo)
  (expect (find-c-translation "bar") 'your-bar))

(deftest (translate-c-name)
  (clear-c-name-translations)
  (expect (translate-c-name "foo") 'foo)
  (expect (translate-c-name "bar") 'bar)
  (define-c-translation "foo" 'my-foo)
  (expect (translate-c-name "foo") 'my-foo)
  (expect (translate-c-name "bar") 'bar)
  (expect (translate-c-name "ThisIsSomeLongName")
	  'this-is-some-long-name))

(deftest (tokenize)
  (expect (tokenize "123") 123)
  (expect (tokenize ()) ())
  (expect (tokenize '("123")) 123)
  (expect (tokenize '("0xA")) 10))

(deftest (parse-macro)
  (expect
   (parse-macro '(macro ("/usr/include//GL/gl.h" 609) "GL_TEXTURE_WIDTH" "0x1000"))
   '(defconstant +gl-texture-width+ 4096)
   :test #'equal)
  (expect
   (parse-macro '(macro ("/usr/include//GL/gl.h" 469) "GL_MAX_TEXTURE_SIZE" "0x0D33"))
   '(defconstant +gl-max-texture-size+ 3379)
   :test #'equal)
  (expect
   (parse-macro '(macro ("/usr/include/X11/X.h" 142) "NoSymbol" "0L"))
   '(defconstant +no-symbol+ 0)
   :test #'equal))


#||
(show-results (run-tests 'c-interface))
||#
